library mt4_timezone_support;
{$mode objfpc}{$H+} {$calling stdcall}

uses
	CMem,
	Classes,
	{ you can add units after this }
	Windows,
	SysUtils,
	mt4_timezone_support_class,
        mt4_timezone_support_seeklist_class
;

// log a message to the debug monitor
procedure Log(AMessage: String);
begin
	OutputDebugString(PChar(AMessage));
end;

// log a formatted message to the debug monitor
procedure Log(AMessage: String; AArgs: array of const);
begin
  	Log(Format(AMessage, AArgs));
end;

function TZSupportInit(pTZInfoFileName : PChar): LongInt;
var
  	rc: TZSupportClass;
begin
	Log('TZSupportInit: ' + pTZInfoFileName);
	rc     := TZSupportClass.Create(pTZInfoFileName);
	Log('TZSupportInit: Handle=%d', [LongInt(rc)]);
	Result := Longint(rc);
end;


// return true if the FXProClass object belonging to the handle is valid
function isValid(AHandle: LongInt): Boolean;
var
  	Dummy: Boolean;
begin
	try
		// cast the "handle" back into the pointer that it was
		// and then just try to access some field
		Dummy  := TZSupportClass(AHandle).isValid;
		Result := True;
	except
		// if invalid this should have raised an access violation
		Log('Invalid (not existing) handle for TZSupportClass object: %d', [AHandle]);
		Dummy  := Dummy; // suppress the compiler hint about unused variable
		Result := False;
	end;
end;


procedure TZSupportDeInit(AHandle: LongInt);
begin
	Log('TZSupportDeInit: Handle=%d', [AHandle]);
	if isValid(AHandle) then
		TZSupportClass(AHandle).Free;
end;


function StrToBrokerTime(
	AHandle			:LongInt;
        pCurrentBrokerDate	:LongInt;
        pTimeStr		:pChar;
        pBrokerTZ		:pChar
):LONG;
begin
  	// Log('StrToBrokerTime: Invoking Object using hd=%d', [AHandle]);
	if (isValid(AHandle)) then
                exit(
                	TZSupportClass(AHandle).StrToBrokerTime(
                        	pCurrentBrokerDate,
                                pTimeStr,
                                pBrokerTZ)
                )
        else begin
                Log('StrToBrokerTime: Invalid object handle hd=%d', [AHandle]);
        	exit(-1);
        end;
end;

type
  	tTimePair= Array[0..1] of LongInt;

function isTimeBetweenRangeDLL(
        AHandle			: LongInt;
        pCurrentBrokerTime	: LongInt;
        var pTimePair		: tTimePair;
        pTimeFromStr		: pChar;
        pTimeUntilStr		: pChar;
        pBrokerTZStr		: pChar
):Integer;
var
        rc	:Integer;
begin
  	// Log('isTimeBetweenRangeDLL: Invoking Object using hd=%d', [AHandle]);
	if (isValid(AHandle)) then begin
        	rc:=TZSupportClass(AHandle).isTimeBetweenRange(
                	pCurrentBrokerTime,
                	pTimePair[0],
                        pTimePair[1],
                        pTimeFromStr,
                        pTimeUntilStr,
                        pBrokerTZStr
		);
                exit(rc);
        end else begin
                Log('isTimeBetweenRangeDLL: Invalid object handle hd=%d', [AHandle]);
        	exit(-1);
        end;
end;

exports
  	TZSupportInit,
  	TZSupportDeInit,
  	StrToBrokerTime,
        isTimeBetweenRangeDLL
;
end.

