unit mt4_timezone_support_class;

{$mode objfpc}{$H+}

interface

uses
	CMem,
	Classes,
	Windows,
	DateUtils,
	SysUtils,
	StrUtils,
	// Pascal-Timezones (source: http://wiki.freepascal.org/PascalTZ)
	upascaltz,
        mt4_timezone_support_seeklist_class;
type
	TZSupportClass = class
		protected
                	valid  			: boolean;
		        tzInfo 			: TPascalTZ;
                        seekCache		: TDateSeekList;
			procedure Msg(ALevel: Integer; AMethod: Ansistring; AMessage: Ansistring);
			procedure Msg(ALevel: Integer; AMethod: Ansistring; AMessage: Ansistring; AArgs: array of const);
		public
			// start an instance of this class
			constructor Create(pTZInfoFileName : AnsiString); reintroduce;

			// stop the instance of this class. Call this before Free
			procedure Stop;

			// terminate the instance (don't call this, call Free as usual)
			destructor Destroy; override;

		        // Calculate the broker time by a time string and the broker timezone
		        function StrToBrokerTime(
		   		pCurrentBrokerDate	: LongInt;
		                pTimeStr		: AnsiString;
		                pBrokerTZ		: AnsiString
		        ):LONG;

		        // Returns TRUE when the pBrokerTime is between pTimeFrom and pTimeUntil
		        // Sets pTimeFrom and pTimeUntil to the respective Posix Timestamps
		        // according to pTimeFromStr and pTimeUntilStr
		        function isTimeBetweenRange(
		                pCurrentBrokerTime	: LongInt;
		                var	pTimeFrom	: LongInt;
				var	pTimeUntil	: LongInt;
		                pTimeFromStr		: AnsiString;
		                pTimeUntilStr		: AnsiString;
		                pBrokerTZ		: AnsiString
			):integer;
                        property isValid:boolean read valid;
	end;
        pTZSupportClass = ^TZSupportClass;

implementation

constructor TZSupportClass.Create(pTZInfoFileName : AnsiString);
// var
	// i	:Integer;
        // t	:TStringList;
begin
	// inherited Create(nil);
	Msg(0, 'TZSupportClass', 'Creating');
        try
                Msg(0, 'TZSupportClass', 'Creating tzInfo object');
        	tzInfo := TPascalTZ.Create();
		tzInfo.ParseDataBaseFromFile(pTZInfoFileName);
                {
                t:=TStringList.Create;
                tzInfo.GetTimeZoneNames(t);
                t.Sort;
                for i:=0 to t.Count-1 do Begin
			Msg(0, 'TZSupportClass',
                        	'tz#%d: "%s"',
                                [i, t.Strings[i]]
                        );
                end;
                t.Destroy;
                }
                Msg(0, 'TZSupportClass',
                	'Created tzInfo object successfully (%d lines read), hd=%d',
                        [tzInfo.ProcessedLines, longInt(self)]
                );
        except on E:Exception do Begin
                Msg(0, 'TZSupportClass', 'ERROR while creating tzInfo object');
                tzInfo.Destroy();
                tzInfo:=NIL;
        	end;
        end;
	seekCache := TDateSeekList.Create;

end;

procedure TZSupportClass.Stop;
begin
  	Msg(-1, 'TZSupportClass', 'Destroying');
        Msg(-1, 'TZSupportClass', 'Destroyed');
end;

destructor TZSupportClass.Destroy;
begin
  self.Stop;
  if (tzInfo<>NIL) then
          tzInfo.Destroy;
  if (seekCache<>NIL) then
          seekCache.Destroy;
  inherited Destroy;
end;


procedure TZSupportClass.Msg(ALevel: Integer; AMethod: Ansistring; AMessage: Ansistring);
begin
	AMessage := '<' + IntToStr(ALevel) + '> ' + AMethod + ': ' + AMessage;
	{$ifdef win32}
	OutputDebugString(PChar(AMessage));
	{$else}
	writeln(AMessage);
	{$endif}
end;

procedure TZSupportClass.Msg(ALevel: Integer; AMethod: Ansistring; AMessage: Ansistring; AArgs: array of const);
begin
	try
		Msg(ALevel, AMethod, Format(AMessage, AArgs));
	except
		Msg(ALevel, AMethod, 'Could not format message' );
	end;
end;

function TZSupportClass.StrToBrokerTime(
	pCurrentBrokerDate	: LongInt;
        pTimeStr		: AnsiString;
        pBrokerTZ		: AnsiString
):LONG;
var
        p	 	: AnsiString;
        timeStr  	: AnsiString;
	tzStr	 	: AnsiString;
        tzSubFix 	: AnsiString;
        timeHour 	: AnsiString;
        timeMin  	: AnsiString;
        i,j	 	: Integer;
        rc	 	: TDateTime;
        rc_Long		: LongInt;
        BrokerTime	: TDateTime;
        found		: boolean;
begin
  	found := false;
	try
        	{
                Msg(0, 'StrToBrokerTime',
	        	'Invoking: par=("%s", "%s") hd=%d',
                        [pTimeStr, pBrokerTZ, LongInt(self)]
                );
                }
		timeStr:=trim(pTimeStr);
	        if (length(timeStr)=0) then
	        	exit(-1);
                BrokerTime := DateOf(UnixToDateTime(pCurrentBrokerDate));
                // TODO: Check whether we've already calculated this date
                // to avoid costly searches in the TZ Database.
                rc_Long := seekCache.seekTimeStamp(BrokerTime, pTimeStr, pBrokerTZ);
                if (rc_Long=-1) then begin
		        // Split the time zone information from the time string
		        tzStr := '';
		        timeHour := '';
		        timeMin := '';
		        // We expect a time string "hh24:mi timezone"
		        i:=1;
		        j:=1;
		        p:=timeStr;
		        while (p[i]>='0') and (p[i]<='9') do begin
		                i:=i+1;
		        end;
		        timeHour:=midStr(p, 1, i-j);
		        i:=i+1;
		        j:=i;
		        while (p[i]>='0') and (p[i]<='9') do begin
		                i:=i+1;
		        end;
		        timeMin:=midStr(p, j, i-j);
		        if (i<length(p)) then begin
		                tzStr:=trim(midStr(p, i, length(p) - i + 1));
		        end;
		        p:=timeHour+':'+timeMin;
	                {
	                Msg(0, 'StrToBrokerTime',
		        	'Time: ' + p + ' ' + tzStr
	                );
	                }
	                // RC now contains a timezone free representation.
		        rc:=StrToDateTime(p);
		        //
		        // Add the current date from the broker
		        rc:= rc + BrokerTime;
	                {
	                Msg(0, 'StrToBrokerTime',
		        	'Time OLD: ' + DateTimeToStr(rc) + ' ' + tzStr
	                );
	                }
		        // and recalculate rc to the target time zone
			if (length(tzStr)>0) and (tzInfo<>NIL) then begin
		                rc:=tzInfo.TimeZoneToTimeZone(rc, tzStr, pBrokerTZ, tzSubFix);
		        end;
	                {
	                Msg(0, 'StrToBrokerTime',
		        	'Time NEW: ' + DateTimeToStr(rc) + ' ' + pBrokerTZ
	                );
	                }
	                rc_Long := LONG(Round((rc - 25569) * 86400));
                end else begin
                        found := true;
                end;
                if (not found) then begin
			seekCache.addTimeStamp(BrokerTime, pTimeStr, pBrokerTZ, rc_Long);
                end;
		// Return a Posix-Timestamp compatible with MT4
	        exit( rc_Long );
        except on E: Exception do begin
		        Msg(0, 'StrToBrokerTime',
		        	'Error calculating broker time par=("' + pTimeStr + '", "' + pBrokerTZ + '")'
		        );
                        Msg(0, 'StrToBrokertime',
                        	'Error %s (%d lines) hd=%d', [E.Message, tzInfo.ProcessedLines, longint(self)]
                        );
		  	exit(-1);

                end;
        end;
end;

function TZSupportClass.isTimeBetweenRange(
        pCurrentBrokerTime	: LongInt;
        var	pTimeFrom	: LongInt;
	var	pTimeUntil	: LongInt;
        pTimeFromStr		: AnsiString;
        pTimeUntilStr		: AnsiString;
        pBrokerTZ		: AnsiString
):integer;
var
        rc	:integer;
        t	:LongInt;
begin
  	{
        Msg(0, 'isTimeBetweenRange',
        	'Invoking: par=("%s", "%s", "%S") hd=%d',
                [pTimeFromStr, pTimeUntilStr, pBrokerTZ, LongInt(self)]
        );
        }
        t:=StrToBrokerTime(pCurrentBrokerTime, pTimeFromStr, pBrokerTZ);
	{
        Msg(0, 'isTimeBetweenRange',
        	'Setting pTimeFrom to  %d ',[t]
        );
        }
        pTimeFrom  := t;
        {
        Msg(0, 'isTimeBetweenRange',
        	'Invoking CalculateBrokerTime: par=("' + pTimeUntilStr + '", "' + pBrokerTZ + '")'
        );
        }
        t:=StrToBrokerTime(pCurrentBrokerTime, pTimeUntilStr, pBrokerTZ);
        {
        Msg(0, 'isTimeBetweenRange',
        	'Setting pTimeUntil to %d ',[t]
        );
        }
        pTimeUntil :=t;
        // Add 24 hours to the UNTIL-Time to get a valid range
        if (pTimeFrom>pTimeUntil) then begin
                pTimeUntil:=pTimeUntil + 3600*24;
        end;
	if ( (pCurrentBrokerTime>=pTimeFrom) and (pCurrentBrokerTime<pTimeUntil) ) then begin
                rc:=1;
        end else begin
        	rc:=0;
        end;
        {
        Msg(0, 'isTimeBetweenRange',
        	'rc=%d ("%s", "%s") hd=%d',
                [rc, pTimeUntilStr, pBrokerTZ, LongInt(self)]
        );
        }
        exit(rc);
end;



end.

